#
#
#

from importlib import import_module

from keyring import get_keyring

from octodns.secret.base import BaseSecrets
from octodns.secret.exception import SecretsException

# TODO: remove __VERSION__ with the next major version release
__version__ = __VERSION__ = '0.0.0'


class KeyringSecretsException(SecretsException):
    pass


class KeyringSecretsBackendException(KeyringSecretsException):
    pass


class KeyringSecrets(BaseSecrets):
    def __init__(self, name, backend=None):
        super().__init__(name)
        self.backend = self._load_backend(backend)

    def _load_backend(self, backend):
        if backend is None:
            return get_keyring()

        try:
            module_name, class_name = backend.rsplit('.', 1)
            module = import_module(module_name)
        except (ImportError, ValueError):
            self.log.exception(
                '_load_backend: Unable to import module "%s"', backend
            )
            raise KeyringSecretsBackendException(
                f'Unknown backend class: "{backend}"'
            )

        try:
            klass = getattr(module, class_name)
        except AttributeError:
            self.log.exception(
                '__init__: Unable to get class "%s" from module "%s"',
                class_name,
                module,
            )
            raise KeyringSecretsBackendException(
                f'Unknown backend class: "{backend}"'
            )

        return klass()

    def fetch(self, name, source):
        service_name, secret_name = name.split('/')
        ret = self.backend.get_password(service_name, secret_name)
        if ret is None:
            raise KeyringSecretsException(f'failed to find {name}')
        return ret



#
#
#

from unittest import TestCase

from octodns_keyring import (
    KeyringSecrets,
    KeyringSecretsBackendException,
    KeyringSecretsException,
)


class TestKeyringSecrets(TestCase):
    def test_keyring(self):
        ks = KeyringSecrets('test')
        self.assertTrue(ks.backend)
        # we have no idea what secrets this one will have since it's
        # auto-selected by the code based on OS and such

        with self.assertRaises(KeyringSecretsBackendException) as ctx:
            KeyringSecrets('test', backend='')
        self.assertEqual('Unknown backend class: ""', str(ctx.exception))

        with self.assertRaises(KeyringSecretsBackendException) as ctx:
            KeyringSecrets('test', backend='does.not.Exist')
        self.assertEqual(
            'Unknown backend class: "does.not.Exist"', str(ctx.exception)
        )

        with self.assertRaises(KeyringSecretsBackendException) as ctx:
            KeyringSecrets('test', backend='helpers.NoSuchClass')
        self.assertEqual(
            'Unknown backend class: "helpers.NoSuchClass"', str(ctx.exception)
        )

        ks = KeyringSecrets('test', backend='helpers.DummyBackend')
        # not importing and using the class object here so we don't help the
        # code by having everything imported beforhand
        self.assertEqual('DummyBackend', ks.backend.__class__.__name__)
        self.assertEqual(
            'dummy-octodns-the-secret', ks.fetch('octodns/the-secret', None)
        )

        ks = KeyringSecrets('test', backend='submod.helpers.SubmodBackend')
        # same here
        self.assertEqual('SubmodBackend', ks.backend.__class__.__name__)
        self.assertEqual(
            'submod-octodns-the-secret', ks.fetch('octodns/the-secret', None)
        )

        ks = KeyringSecrets('test', backend='helpers.NoneBackend')
        # same here
        self.assertEqual('NoneBackend', ks.backend.__class__.__name__)
        with self.assertRaises(KeyringSecretsException) as ctx:
            ks.fetch('octodns/the-secret', None)
        self.assertEqual(
            'failed to find octodns/the-secret', str(ctx.exception)
        )
